from django.apps import AppConfig
from django.conf import settings

from health_check.plugins import plugin_dir


class HealthCheckConfig(AppConfig):
    name = "health_check.contrib.psutil"

    def ready(self):
        from .backends import DiskUsage, MemoryUsage

        # Ensure checks haven't been explicitly disabled before registering
        if (
            hasattr(settings, "HEALTH_CHECK")
            and ("DISK_USAGE_MAX" in settings.HEALTH_CHECK)
            and (settings.HEALTH_CHECK["DISK_USAGE_MAX"] is None)
        ):
            pass
        else:
            plugin_dir.register(DiskUsage)
        if (
            hasattr(settings, "HEALTH_CHECK")
            and ("DISK_USAGE_MAX" in settings.HEALTH_CHECK)
            and (settings.HEALTH_CHECK["MEMORY_MIN"] is None)
        ):
            pass
        else:
            plugin_dir.register(MemoryUsage)



from django.apps import AppConfig

from health_check.plugins import plugin_dir


class HealthCheckConfig(AppConfig):
    default_auto_field = "django.db.models.AutoField"
    name = "health_check.db"

    def ready(self):
        from .backends import DatabaseBackend

        plugin_dir.register(DatabaseBackend)



import json

import pytest

from health_check.backends import BaseHealthCheckBackend
from health_check.conf import HEALTH_CHECK
from health_check.exceptions import ServiceWarning
from health_check.plugins import plugin_dir
from health_check.views import MediaType

try:
    from django.urls import reverse
except ImportError:
    from django.core.urlresolvers import reverse


class TestMediaType:
    def test_lt(self):
        assert not MediaType("*/*") < MediaType("*/*")
        assert not MediaType("*/*") < MediaType("*/*", 0.9)
        assert MediaType("*/*", 0.9) < MediaType("*/*")

    def test_str(self):
        assert str(MediaType("*/*")) == "*/*; q=1.0"
        assert str(MediaType("image/*", 0.6)) == "image/*; q=0.6"

    def test_repr(self):
        assert repr(MediaType("*/*")) == "MediaType: */*; q=1.0"

    def test_eq(self):
        assert MediaType("*/*") == MediaType("*/*")
        assert MediaType("*/*", 0.9) != MediaType("*/*")

    valid_strings = [
        ("*/*", MediaType("*/*")),
        ("*/*; q=0.9", MediaType("*/*", 0.9)),
        ("*/*; q=0", MediaType("*/*", 0.0)),
        ("*/*; q=0.0", MediaType("*/*", 0.0)),
        ("*/*; q=0.1", MediaType("*/*", 0.1)),
        ("*/*; q=0.12", MediaType("*/*", 0.12)),
        ("*/*; q=0.123", MediaType("*/*", 0.123)),
        ("*/*; q=1.000", MediaType("*/*", 1.0)),
        ("*/*; q=1", MediaType("*/*", 1.0)),
        ("*/*;q=0.9", MediaType("*/*", 0.9)),
        ("*/* ;q=0.9", MediaType("*/*", 0.9)),
        ("*/* ; q=0.9", MediaType("*/*", 0.9)),
        ("*/* ;   q=0.9", MediaType("*/*", 0.9)),
        ("*/*;v=b3", MediaType("*/*")),
        ("*/*; q=0.5; v=b3", MediaType("*/*", 0.5)),
    ]

    @pytest.mark.parametrize("type, expected", valid_strings)
    def test_from_valid_strings(self, type, expected):
        assert MediaType.from_string(type) == expected

    invalid_strings = [
        "*/*;0.9",
        'text/html;z=""',
        "text/html; xxx",
        "text/html;  =a",
    ]

    @pytest.mark.parametrize("type", invalid_strings)
    def test_from_invalid_strings(self, type):
        with pytest.raises(ValueError) as e:
            MediaType.from_string(type)
        expected_error = '"%s" is not a valid media type' % type
        assert expected_error in str(e.value)

    def test_parse_header(self):
        assert list(MediaType.parse_header()) == [
            MediaType("*/*"),
        ]
        assert list(
            MediaType.parse_header(
                "text/html; q=0.1, application/xhtml+xml; q=0.1 ,application/json"
            )
        ) == [
            MediaType("application/json"),
            MediaType("text/html", 0.1),
            MediaType("application/xhtml+xml", 0.1),
        ]


class TestMainView:
    url = reverse("health_check:health_check_home")

    def test_success(self, client):
        response = client.get(self.url)
        assert response.status_code == 200, response.content.decode("utf-8")
        assert response["content-type"] == "text/html; charset=utf-8"

    def test_error(self, client):
        class MyBackend(BaseHealthCheckBackend):
            def check_status(self):
                self.add_error("Super Fail!")

        plugin_dir.reset()
        plugin_dir.register(MyBackend)
        response = client.get(self.url)
        assert response.status_code == 500, response.content.decode("utf-8")
        assert response["content-type"] == "text/html; charset=utf-8"
        assert b"Super Fail!" in response.content

    def test_warning(self, client):
        class MyBackend(BaseHealthCheckBackend):
            def check_status(self):
                raise ServiceWarning("so so")

        plugin_dir.reset()
        plugin_dir.register(MyBackend)
        response = client.get(self.url)
        assert response.status_code == 500, response.content.decode("utf-8")
        assert b"so so" in response.content, response.content

        HEALTH_CHECK["WARNINGS_AS_ERRORS"] = False

        response = client.get(self.url)
        assert response.status_code == 200, response.content.decode("utf-8")
        assert response["content-type"] == "text/html; charset=utf-8"
        assert b"so so" in response.content, response.content

    def test_non_critical(self, client):
        class MyBackend(BaseHealthCheckBackend):
            critical_service = False

            def check_status(self):
                self.add_error("Super Fail!")

        plugin_dir.reset()
        plugin_dir.register(MyBackend)
        response = client.get(self.url)
        assert response.status_code == 200, response.content.decode("utf-8")
        assert response["content-type"] == "text/html; charset=utf-8"
        assert b"Super Fail!" in response.content

    def test_success_accept_json(self, client):
        class JSONSuccessBackend(BaseHealthCheckBackend):
            def run_check(self):
                pass

        plugin_dir.reset()
        plugin_dir.register(JSONSuccessBackend)
        response = client.get(self.url, HTTP_ACCEPT="application/json")
        assert response["content-type"] == "application/json"
        assert response.status_code == 200

    def test_success_prefer_json(self, client):
        class JSONSuccessBackend(BaseHealthCheckBackend):
            def run_check(self):
                pass

        plugin_dir.reset()
        plugin_dir.register(JSONSuccessBackend)
        response = client.get(
            self.url, HTTP_ACCEPT="application/json; q=0.8, text/html; q=0.5"
        )
        assert response["content-type"] == "application/json"
        assert response.status_code == 200

    def test_success_accept_xhtml(self, client):
        class SuccessBackend(BaseHealthCheckBackend):
            def run_check(self):
                pass

        plugin_dir.reset()
        plugin_dir.register(SuccessBackend)
        response = client.get(self.url, HTTP_ACCEPT="application/xhtml+xml")
        assert response["content-type"] == "text/html; charset=utf-8"
        assert response.status_code == 200

    def test_success_unsupported_accept(self, client):
        class SuccessBackend(BaseHealthCheckBackend):
            def run_check(self):
                pass

        plugin_dir.reset()
        plugin_dir.register(SuccessBackend)
        response = client.get(self.url, HTTP_ACCEPT="application/octet-stream")
        assert response["content-type"] == "text/plain"
        assert response.status_code == 406
        assert (
            response.content
            == b"Not Acceptable: Supported content types: text/html, application/json"
        )

    def test_success_unsupported_and_supported_accept(self, client):
        class SuccessBackend(BaseHealthCheckBackend):
            def run_check(self):
                pass

        plugin_dir.reset()
        plugin_dir.register(SuccessBackend)
        response = client.get(
            self.url, HTTP_ACCEPT="application/octet-stream, application/json; q=0.9"
        )
        assert response["content-type"] == "application/json"
        assert response.status_code == 200

    def test_success_accept_order(self, client):
        class JSONSuccessBackend(BaseHealthCheckBackend):
            def run_check(self):
                pass

        plugin_dir.reset()
        plugin_dir.register(JSONSuccessBackend)
        response = client.get(
            self.url,
            HTTP_ACCEPT="text/html, application/xhtml+xml, application/json; q=0.9, */*; q=0.1",
        )
        assert response["content-type"] == "text/html; charset=utf-8"
        assert response.status_code == 200

    def test_success_accept_order__reverse(self, client):
        class JSONSuccessBackend(BaseHealthCheckBackend):
            def run_check(self):
                pass

        plugin_dir.reset()
        plugin_dir.register(JSONSuccessBackend)
        response = client.get(
            self.url,
            HTTP_ACCEPT="text/html; q=0.1, application/xhtml+xml; q=0.1, application/json",
        )
        assert response["content-type"] == "application/json"
        assert response.status_code == 200

    def test_format_override(self, client):
        class JSONSuccessBackend(BaseHealthCheckBackend):
            def run_check(self):
                pass

        plugin_dir.reset()
        plugin_dir.register(JSONSuccessBackend)
        response = client.get(self.url + "?format=json", HTTP_ACCEPT="text/html")
        assert response["content-type"] == "application/json"
        assert response.status_code == 200

    def test_format_no_accept_header(self, client):
        class JSONSuccessBackend(BaseHealthCheckBackend):
            def run_check(self):
                pass

        plugin_dir.reset()
        plugin_dir.register(JSONSuccessBackend)
        response = client.get(self.url)
        assert response.status_code == 200, response.content.decode("utf-8")
        assert response["content-type"] == "text/html; charset=utf-8"

    def test_error_accept_json(self, client):
        class JSONErrorBackend(BaseHealthCheckBackend):
            def run_check(self):
                self.add_error("JSON Error")

        plugin_dir.reset()
        plugin_dir.register(JSONErrorBackend)
        response = client.get(self.url, HTTP_ACCEPT="application/json")
        assert response.status_code == 500, response.content.decode("utf-8")
        assert response["content-type"] == "application/json"
        assert (
            "JSON Error"
            in json.loads(response.content.decode("utf-8"))[
                JSONErrorBackend().identifier()
            ]
        )

    def test_success_param_json(self, client):
        class JSONSuccessBackend(BaseHealthCheckBackend):
            def run_check(self):
                pass

        plugin_dir.reset()
        plugin_dir.register(JSONSuccessBackend)
        response = client.get(self.url, {"format": "json"})
        assert response.status_code == 200, response.content.decode("utf-8")
        assert response["content-type"] == "application/json"
        assert json.loads(response.content.decode("utf-8")) == {
            JSONSuccessBackend().identifier(): JSONSuccessBackend().pretty_status()
        }

    def test_success_subset_define(self, client):
        class SuccessOneBackend(BaseHealthCheckBackend):
            def run_check(self):
                pass

        class SuccessTwoBackend(BaseHealthCheckBackend):
            def run_check(self):
                pass

        plugin_dir.reset()
        plugin_dir.register(SuccessOneBackend)
        plugin_dir.register(SuccessTwoBackend)

        HEALTH_CHECK["SUBSETS"] = {
            "startup-probe": ["SuccessOneBackend", "SuccessTwoBackend"],
            "liveness-probe": ["SuccessTwoBackend"],
        }

        response_startup_probe = client.get(
            self.url + "startup-probe/", {"format": "json"}
        )
        assert (
            response_startup_probe.status_code == 200
        ), response_startup_probe.content.decode("utf-8")
        assert response_startup_probe["content-type"] == "application/json"
        assert json.loads(response_startup_probe.content.decode("utf-8")) == {
            SuccessOneBackend().identifier(): SuccessOneBackend().pretty_status(),
            SuccessTwoBackend().identifier(): SuccessTwoBackend().pretty_status(),
        }

        response_liveness_probe = client.get(
            self.url + "liveness-probe/", {"format": "json"}
        )
        assert (
            response_liveness_probe.status_code == 200
        ), response_liveness_probe.content.decode("utf-8")
        assert response_liveness_probe["content-type"] == "application/json"
        assert json.loads(response_liveness_probe.content.decode("utf-8")) == {
            SuccessTwoBackend().identifier(): SuccessTwoBackend().pretty_status(),
        }

    def test_error_subset_not_found(self, client):
        plugin_dir.reset()
        response = client.get(self.url + "liveness-probe/", {"format": "json"})
        print(f"content: {response.content}")
        print(f"code: {response.status_code}")
        assert response.status_code == 404, response.content.decode("utf-8")

    def test_error_param_json(self, client):
        class JSONErrorBackend(BaseHealthCheckBackend):
            def run_check(self):
                self.add_error("JSON Error")

        plugin_dir.reset()
        plugin_dir.register(JSONErrorBackend)
        response = client.get(self.url, {"format": "json"})
        assert response.status_code == 500, response.content.decode("utf-8")
        assert response["content-type"] == "application/json"
        assert (
            "JSON Error"
            in json.loads(response.content.decode("utf-8"))[
                JSONErrorBackend().identifier()
            ]
        )



import copy
from collections import OrderedDict
from concurrent.futures import ThreadPoolExecutor

from django.http import Http404

from health_check.conf import HEALTH_CHECK
from health_check.exceptions import ServiceWarning
from health_check.plugins import plugin_dir


class CheckMixin:
    _errors = None
    _plugins = None

    @property
    def errors(self):
        if not self._errors:
            self._errors = self.run_check()
        return self._errors

    def check(self, subset=None):
        return self.run_check(subset=subset)

    @property
    def plugins(self):
        if not plugin_dir._registry:
            return OrderedDict({})

        if not self._plugins:
            registering_plugins = (
                plugin_class(**copy.deepcopy(options))
                for plugin_class, options in plugin_dir._registry
            )
            registering_plugins = sorted(
                registering_plugins, key=lambda plugin: plugin.identifier()
            )
            self._plugins = OrderedDict(
                {plugin.identifier(): plugin for plugin in registering_plugins}
            )
        return self._plugins

    def filter_plugins(self, subset=None):
        if subset is None:
            return self.plugins

        health_check_subsets = HEALTH_CHECK["SUBSETS"]
        if subset not in health_check_subsets or not self.plugins:
            raise Http404(f"Subset: '{subset}' does not exist.")

        selected_subset = set(health_check_subsets[subset])
        return {
            plugin_identifier: v
            for plugin_identifier, v in self.plugins.items()
            if plugin_identifier in selected_subset
        }

    def run_check(self, subset=None):
        errors = []

        def _run(plugin):
            plugin.run_check()
            try:
                return plugin
            finally:
                from django.db import connections

                connections.close_all()

        def _collect_errors(plugin):
            if plugin.critical_service:
                if not HEALTH_CHECK["WARNINGS_AS_ERRORS"]:
                    errors.extend(
                        e for e in plugin.errors if not isinstance(e, ServiceWarning)
                    )
                else:
                    errors.extend(plugin.errors)

        plugins = self.filter_plugins(subset=subset)
        plugin_instances = plugins.values()

        if HEALTH_CHECK["DISABLE_THREADING"]:
            for plugin in plugin_instances:
                _run(plugin)
                _collect_errors(plugin)
        else:
            with ThreadPoolExecutor(max_workers=len(plugin_instances) or 1) as executor:
                for plugin in executor.map(_run, plugin_instances):
                    _collect_errors(plugin)
        return errors



try:
    import sphinx_rtd_theme
except ImportError:
    sphinx_rtd_theme = None

master_doc = "index"

if sphinx_rtd_theme:
    html_theme = "sphinx_rtd_theme"
    html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]



from celery.exceptions import TaskRevokedError, TimeoutError
from django.conf import settings

from health_check.backends import BaseHealthCheckBackend
from health_check.exceptions import ServiceReturnedUnexpectedResult, ServiceUnavailable

from .tasks import add


class CeleryHealthCheck(BaseHealthCheckBackend):
    def check_status(self):
        timeout = getattr(settings, "HEALTHCHECK_CELERY_TIMEOUT", 3)
        result_timeout = getattr(settings, "HEALTHCHECK_CELERY_RESULT_TIMEOUT", timeout)
        queue_timeout = getattr(settings, "HEALTHCHECK_CELERY_QUEUE_TIMEOUT", timeout)

        try:
            result = add.apply_async(
                args=[4, 4], expires=queue_timeout, queue=self.queue
            )
            result.get(timeout=result_timeout)
            if result.result != 8:
                self.add_error(
                    ServiceReturnedUnexpectedResult("Celery returned wrong result")
                )
        except IOError as e:
            self.add_error(ServiceUnavailable("IOError"), e)
        except NotImplementedError as e:
            self.add_error(
                ServiceUnavailable(
                    "NotImplementedError: Make sure CELERY_RESULT_BACKEND is set"
                ),
                e,
            )
        except TaskRevokedError as e:
            self.add_error(
                ServiceUnavailable(
                    "TaskRevokedError: The task was revoked, likely because it spent "
                    "too long in the queue"
                ),
                e,
            )
        except TimeoutError as e:
            self.add_error(
                ServiceUnavailable(
                    "TimeoutError: The task took too long to return a result"
                ),
                e,
            )
        except BaseException as e:
            self.add_error(ServiceUnavailable("Unknown error"), e)



import logging

from health_check.storage.backends import StorageHealthCheck


class S3BotoStorageHealthCheck(StorageHealthCheck):
    """
    Tests the status of a `S3BotoStorage` file storage backend.

    S3BotoStorage is included in the `django-storages` package
    and recommended by for example Amazon and Heroku for Django
    static and media file storage on cloud platforms.

    ``django-storages`` can be found at https://git.io/v1lGx
    ``S3BotoStorage`` can be found at https://git.io/v1lGF
    """

    logger = logging.getLogger(__name__)
    storage = "storages.backends.s3boto.S3BotoStorage"

    def check_delete(self, file_name):
        storage = self.get_storage()
        storage.delete(file_name)



from unittest.mock import patch

import pytest

from health_check.backends import BaseHealthCheckBackend
from health_check.conf import HEALTH_CHECK
from health_check.mixins import CheckMixin
from health_check.plugins import plugin_dir


class FailPlugin(BaseHealthCheckBackend):
    def check_status(self):
        self.add_error("Oops")


class OkPlugin(BaseHealthCheckBackend):
    def check_status(self):
        pass


class Checker(CheckMixin):
    pass


class TestCheckMixin:
    @pytest.fixture(autouse=True)
    def setup(self):
        plugin_dir.reset()
        plugin_dir.register(FailPlugin)
        plugin_dir.register(OkPlugin)
        yield
        plugin_dir.reset()

    @pytest.mark.parametrize("disable_threading", [(True,), (False,)])
    def test_plugins(self, monkeypatch, disable_threading):
        monkeypatch.setitem(HEALTH_CHECK, "DISABLE_THREADING", disable_threading)

        assert len(Checker().plugins) == 2

    @pytest.mark.parametrize("disable_threading", [(True,), (False,)])
    def test_errors(self, monkeypatch, disable_threading):
        monkeypatch.setitem(HEALTH_CHECK, "DISABLE_THREADING", disable_threading)

        assert len(Checker().errors) == 1

    @pytest.mark.parametrize("disable_threading", [(True,), (False,)])
    def test_run_check(self, monkeypatch, disable_threading):
        monkeypatch.setitem(HEALTH_CHECK, "DISABLE_THREADING", disable_threading)

        assert len(Checker().run_check()) == 1

    def test_run_check_threading_enabled(self, monkeypatch):
        """Ensure threading used when not disabled."""

        # Ensure threading is enabled.
        monkeypatch.setitem(HEALTH_CHECK, "DISABLE_THREADING", False)

        # Ensure ThreadPoolExecutor is used
        with patch("health_check.mixins.ThreadPoolExecutor") as tpe:
            Checker().run_check()
            tpe.assert_called()

    def test_run_check_threading_disabled(self, monkeypatch):
        """Ensure threading not used when disabled."""

        # Ensure threading is disabled.
        monkeypatch.setitem(HEALTH_CHECK, "DISABLE_THREADING", True)

        # Ensure ThreadPoolExecutor is not used
        with patch("health_check.mixins.ThreadPoolExecutor") as tpe:
            Checker().run_check()
            tpe.assert_not_called()



import logging

from django.conf import settings
from redis import exceptions, from_url

from health_check.backends import BaseHealthCheckBackend
from health_check.exceptions import ServiceUnavailable

logger = logging.getLogger(__name__)


class RedisHealthCheck(BaseHealthCheckBackend):
    """Health check for Redis."""

    redis_url = getattr(settings, "REDIS_URL", "redis://localhost/1")

    def check_status(self):
        """Check Redis service by pinging the redis instance with a redis connection."""
        logger.debug("Got %s as the redis_url. Connecting to redis...", self.redis_url)

        logger.debug("Attempting to connect to redis...")
        try:
            # conn is used as a context to release opened resources later
            with from_url(self.redis_url) as conn:
                conn.ping()  # exceptions may be raised upon ping
        except ConnectionRefusedError as e:
            self.add_error(
                ServiceUnavailable(
                    "Unable to connect to Redis: Connection was refused."
                ),
                e,
            )
        except exceptions.TimeoutError as e:
            self.add_error(
                ServiceUnavailable("Unable to connect to Redis: Timeout."), e
            )
        except exceptions.ConnectionError as e:
            self.add_error(
                ServiceUnavailable("Unable to connect to Redis: Connection Error"), e
            )
        except BaseException as e:
            self.add_error(ServiceUnavailable("Unknown error"), e)
        else:
            logger.debug("Connection established. Redis is healthy.")



import os.path
import uuid

from kombu import Queue

BASE_DIR = os.path.dirname(os.path.abspath(__file__))
DEBUG = True

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:",
    },
    "other": {  # 2nd database conneciton to ensure proper connection handling
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":backup:",
    },
}

INSTALLED_APPS = (
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.staticfiles",
    "health_check",
    "health_check.cache",
    "health_check.db",
    "health_check.storage",
    "health_check.contrib.celery",
    "health_check.contrib.migrations",
    "health_check.contrib.celery_ping",
    "health_check.contrib.s3boto_storage",
    "tests",
)

MIDDLEWARE_CLASSES = (
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
)

STATIC_URL = "/static/"

MEDIA_ROOT = os.path.join(BASE_DIR, "media")

SITE_ID = 1
ROOT_URLCONF = "tests.testapp.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "APP_DIRS": True,
        "OPTIONS": {
            "debug": True,
        },
    },
]

SECRET_KEY = uuid.uuid4().hex

USE_TZ = True

CELERY_QUEUES = [
    Queue("default"),
    Queue("queue2"),
]



import unittest
from io import BytesIO
from unittest import mock

import django
from django.core.files.base import File
from django.core.files.storage import Storage
from django.test import TestCase, override_settings

from health_check.contrib.s3boto3_storage.backends import S3Boto3StorageHealthCheck
from health_check.exceptions import ServiceUnavailable
from health_check.storage.backends import (
    DefaultFileStorageHealthCheck,
    StorageHealthCheck,
)


class CustomStorage(Storage):
    pass


class MockStorage(Storage):
    """
    A Mock Storage backend used for testing.
    saves - Determines whether save will mock a successful or unsuccessful save
    deletes -  Determines whether save will mock a successful or unsuccessful deletion
    """

    MOCK_FILE_COUNT = 0
    saves = None
    deletes = None

    def __init__(self, saves=True, deletes=True):
        super(MockStorage, self).__init__()
        self.MOCK_FILE_COUNT = 0
        self.saves = saves
        self.deletes = deletes

    def exists(self, file_name):
        return self.MOCK_FILE_COUNT != 0

    def delete(self, name):
        if self.deletes:
            self.MOCK_FILE_COUNT -= 1

    def save(self, name, content, max_length=None):
        if self.saves:
            self.MOCK_FILE_COUNT += 1


# Mocking the S3Boto3Storage backend
class MockS3Boto3Storage:
    """
    A mock S3Boto3Storage backend to simulate interactions with AWS S3.
    """

    def __init__(self, saves=True, deletes=True):
        self.saves = saves
        self.deletes = deletes
        self.files = {}

    def open(self, name, mode="rb"):
        """
        Simulates opening a file from the mocked S3 storage.
        For simplicity, this doesn't differentiate between read and write modes.
        """
        if name in self.files:
            # Assuming file content is stored as bytes
            file_content = self.files[name]
            if isinstance(file_content, bytes):
                return File(BytesIO(file_content))
            else:
                raise ValueError("File content must be bytes.")
        else:
            raise FileNotFoundError(f"The file {name} does not exist.")

    def save(self, name, content):
        """
        Overriding save to ensure content is stored as bytes in a way compatible with open method.
        Assumes content is either a ContentFile, bytes, or a string that needs conversion.
        """
        if self.saves:
            # Check if content is a ContentFile or similar and read bytes
            if hasattr(content, "read"):
                file_content = content.read()
            elif isinstance(content, bytes):
                file_content = content
            elif isinstance(content, str):
                file_content = content.encode()  # Convert string to bytes
            else:
                raise ValueError("Unsupported file content type.")

            self.files[name] = file_content
            return name
        raise Exception("Failed to save file.")

    def delete(self, name):
        if self.deletes:
            self.files.pop(name, None)
        else:
            raise Exception("Failed to delete file.")

    def exists(self, name):
        return name in self.files


def get_file_name(*args, **kwargs):
    return "mockfile.txt"


def get_file_content(*args, **kwargs):
    return b"mockcontent"


@mock.patch(
    "health_check.storage.backends.StorageHealthCheck.get_file_name", get_file_name
)
@mock.patch(
    "health_check.storage.backends.StorageHealthCheck.get_file_content",
    get_file_content,
)
class HealthCheckStorageTests(TestCase):
    """
    Tests health check behavior with a mocked storage backend.
    Ensures check_status returns/raises the expected result when the storage works or raises exceptions.
    """

    def test_get_storage(self):
        """Test get_storage method returns None on the base class, but a Storage instance on default."""
        base_storage = StorageHealthCheck()
        self.assertIsNone(base_storage.get_storage())

        default_storage = DefaultFileStorageHealthCheck()
        self.assertIsInstance(default_storage.get_storage(), Storage)

    @unittest.skipUnless((4, 2) <= django.VERSION < (5, 0), "Only for Django 4.2 - 5.0")
    def test_get_storage_django_between_42_and_50(self):
        """Check that the old DEFAULT_FILE_STORAGE setting keeps being supported."""
        # Note: this test doesn't work on Django<4.2 because the setting value is
        # evaluated when the class attribute DefaultFileStorageHealthCheck.store is
        # read, which is at import time, before we can mock the setting.
        with self.settings(DEFAULT_FILE_STORAGE="tests.test_storage.CustomStorage"):
            default_storage = DefaultFileStorageHealthCheck()
            self.assertIsInstance(default_storage.get_storage(), CustomStorage)

    @unittest.skipUnless((4, 2) <= django.VERSION, "Django 4.2+ required")
    def test_get_storage_django_42_plus(self):
        """Check that the new STORAGES setting is supported."""
        with self.settings(
            STORAGES={"default": {"BACKEND": "tests.test_storage.CustomStorage"}}
        ):
            default_storage = DefaultFileStorageHealthCheck()
            self.assertIsInstance(default_storage.get_storage(), CustomStorage)

    @mock.patch(
        "health_check.storage.backends.DefaultFileStorageHealthCheck.storage",
        MockStorage(),
    )
    def test_check_status_working(self):
        """Test check_status returns True when storage is working properly."""
        default_storage_health = DefaultFileStorageHealthCheck()

        default_storage = default_storage_health.get_storage()

        default_storage_open = "{}.{}.open".format(
            default_storage.__module__, default_storage.__class__.__name__
        )

        with mock.patch(
            default_storage_open,
            mock.mock_open(read_data=default_storage_health.get_file_content()),
        ):
            self.assertTrue(default_storage_health.check_status())

    @unittest.skipUnless(django.VERSION <= (4, 1), "Only for Django 4.1 and earlier")
    @mock.patch(
        "health_check.storage.backends.DefaultFileStorageHealthCheck.storage",
        MockStorage(saves=False),
    )
    def test_file_does_not_exist_django_41_earlier(self):
        """Test check_status raises ServiceUnavailable when file is not saved."""
        default_storage_health = DefaultFileStorageHealthCheck()
        with self.assertRaises(ServiceUnavailable):
            default_storage_health.check_status()

    @unittest.skipUnless((4, 2) <= django.VERSION, "Only for Django 4.2+")
    @mock.patch(
        "health_check.storage.backends.storages",
        {"default": MockStorage(saves=False)},
    )
    def test_file_does_not_exist_django_42_plus(self):
        """Test check_status raises ServiceUnavailable when file is not saved."""
        default_storage_health = DefaultFileStorageHealthCheck()
        with self.assertRaises(ServiceUnavailable):
            default_storage_health.check_status()

    @unittest.skipUnless(django.VERSION <= (4, 1), "Only for Django 4.1 and earlier")
    @mock.patch(
        "health_check.storage.backends.DefaultFileStorageHealthCheck.storage",
        MockStorage(deletes=False),
    )
    def test_file_not_deleted_django_41_earlier(self):
        """Test check_status raises ServiceUnavailable when file is not deleted."""
        default_storage_health = DefaultFileStorageHealthCheck()
        with self.assertRaises(ServiceUnavailable):
            default_storage_health.check_status()

    @unittest.skipUnless((4, 2) <= django.VERSION, "Only for Django 4.2+")
    @mock.patch(
        "health_check.storage.backends.storages",
        {"default": MockStorage(deletes=False)},
    )
    def test_file_not_deleted_django_42_plus(self):
        """Test check_status raises ServiceUnavailable when file is not deleted."""
        default_storage_health = DefaultFileStorageHealthCheck()
        with self.assertRaises(ServiceUnavailable):
            default_storage_health.check_status()


@mock.patch("storages.backends.s3boto3.S3Boto3Storage", new=MockS3Boto3Storage)
@override_settings(
    STORAGES={
        "default": {"BACKEND": "storages.backends.s3boto3.S3Boto3Storage"},
    }
)
class HealthCheckS3Boto3StorageTests(TestCase):
    """
    Tests health check behavior with a mocked S3Boto3Storage backend.
    """

    @unittest.skipUnless(django.VERSION <= (4, 1), "Only for Django 4.1 and earlier")
    @mock.patch(
        "storages.backends.s3boto3.S3Boto3Storage",
        MockS3Boto3Storage(deletes=False),
    )
    def test_check_delete_success_django_41_earlier(self):
        """Test that check_delete correctly deletes a file when S3Boto3Storage is working."""
        health_check = S3Boto3StorageHealthCheck()
        mock_storage = health_check.get_storage()
        file_name = "testfile.txt"
        content = BytesIO(b"Test content")
        mock_storage.save(file_name, content)

        with self.assertRaises(ServiceUnavailable):
            health_check.check_delete(file_name)

    @unittest.skipUnless((4, 2) <= django.VERSION, "Only for Django 4.2+")
    def test_check_delete_success(self):
        """Test that check_delete correctly deletes a file when S3Boto3Storage is working."""
        health_check = S3Boto3StorageHealthCheck()
        mock_storage = health_check.get_storage()
        file_name = "testfile.txt"
        content = BytesIO(b"Test content")
        mock_storage.save(file_name, content)

        health_check.check_delete(file_name)
        self.assertFalse(mock_storage.exists(file_name))

    def test_check_delete_failure(self):
        """Test that check_delete raises ServiceUnavailable when deletion fails."""
        with mock.patch.object(
            MockS3Boto3Storage,
            "delete",
            side_effect=Exception("Failed to delete file."),
        ):
            health_check = S3Boto3StorageHealthCheck()
            with self.assertRaises(ServiceUnavailable):
                health_check.check_delete("testfile.txt")

    @unittest.skipUnless(django.VERSION <= (4, 1), "Only for Django 4.1 and earlier")
    @mock.patch(
        "storages.backends.s3boto3.S3Boto3Storage",
        MockS3Boto3Storage(deletes=False),
    )
    def test_check_status_working_django_41_earlier(self):
        """Test check_status returns True when S3Boto3Storage can save and delete files."""
        health_check = S3Boto3StorageHealthCheck()
        with self.assertRaises(ServiceUnavailable):
            self.assertTrue(health_check.check_status())

    @unittest.skipUnless((4, 2) <= django.VERSION, "Only for Django 4.2+")
    def test_check_status_working(self):
        """Test check_status returns True when S3Boto3Storage can save and delete files."""
        health_check = S3Boto3StorageHealthCheck()
        self.assertTrue(health_check.check_status())

    def test_check_status_failure_on_save(self):
        """Test check_status raises ServiceUnavailable when file cannot be saved."""
        with mock.patch.object(
            MockS3Boto3Storage, "save", side_effect=Exception("Failed to save file.")
        ):
            health_check = S3Boto3StorageHealthCheck()
            with self.assertRaises(ServiceUnavailable):
                health_check.check_status()

    def test_check_status_failure_on_delete(self):
        """Test check_status raises ServiceUnavailable when file cannot be deleted."""
        with mock.patch.object(
            MockS3Boto3Storage, "exists", new_callable=mock.PropertyMock
        ) as mock_exists:
            mock_exists.return_value = False
            health_check = S3Boto3StorageHealthCheck()
            with self.assertRaises(ServiceUnavailable):
                health_check.check_status()



from django.apps import AppConfig

from health_check.plugins import plugin_dir


class HealthCheckConfig(AppConfig):
    name = "health_check.storage"

    def ready(self):
        from .backends import DefaultFileStorageHealthCheck

        plugin_dir.register(DefaultFileStorageHealthCheck)



from io import StringIO

import pytest
from django.core.management import call_command

from health_check.backends import BaseHealthCheckBackend
from health_check.conf import HEALTH_CHECK
from health_check.plugins import plugin_dir


class FailPlugin(BaseHealthCheckBackend):
    def check_status(self):
        self.add_error("Oops")


class OkPlugin(BaseHealthCheckBackend):
    def check_status(self):
        pass


class TestCommand:
    @pytest.fixture(autouse=True)
    def setup(self):
        plugin_dir.reset()
        plugin_dir.register(FailPlugin)
        plugin_dir.register(OkPlugin)
        yield
        plugin_dir.reset()

    def test_command(self):
        stdout = StringIO()
        with pytest.raises(SystemExit):
            call_command("health_check", stdout=stdout)
        stdout.seek(0)
        assert stdout.read() == (
            "FailPlugin               ... unknown error: Oops\n"
            "OkPlugin                 ... working\n"
        )

    def test_command_with_subset(self):
        SUBSET_NAME_1 = "subset-1"
        SUBSET_NAME_2 = "subset-2"
        HEALTH_CHECK["SUBSETS"] = {
            SUBSET_NAME_1: ["OkPlugin"],
            SUBSET_NAME_2: ["OkPlugin", "FailPlugin"],
        }

        stdout = StringIO()
        call_command("health_check", f"--subset={SUBSET_NAME_1}", stdout=stdout)
        stdout.seek(0)
        assert stdout.read() == ("OkPlugin                 ... working\n")

    def test_command_with_failed_check_subset(self):
        SUBSET_NAME = "subset-2"
        HEALTH_CHECK["SUBSETS"] = {SUBSET_NAME: ["OkPlugin", "FailPlugin"]}

        stdout = StringIO()
        with pytest.raises(SystemExit):
            call_command("health_check", f"--subset={SUBSET_NAME}", stdout=stdout)
        stdout.seek(0)
        assert stdout.read() == (
            "FailPlugin               ... unknown error: Oops\n"
            "OkPlugin                 ... working\n"
        )

    def test_command_with_non_existence_subset(self):
        SUBSET_NAME = "subset-2"
        NON_EXISTENCE_SUBSET_NAME = "abcdef12"
        HEALTH_CHECK["SUBSETS"] = {SUBSET_NAME: ["OkPlugin"]}

        stdout = StringIO()
        with pytest.raises(SystemExit):
            call_command(
                "health_check", f"--subset={NON_EXISTENCE_SUBSET_NAME}", stdout=stdout
            )
        stdout.seek(0)
        assert stdout.read() == (
            f"Subset: '{NON_EXISTENCE_SUBSET_NAME}' does not exist.\n"
        )



from django.apps import AppConfig

from health_check.plugins import plugin_dir


class HealthCheckConfig(AppConfig):
    name = "health_check.contrib.s3boto3_storage"

    def ready(self):
        from .backends import S3Boto3StorageHealthCheck

        plugin_dir.register(S3Boto3StorageHealthCheck)



from unittest import mock

from redis.exceptions import ConnectionError, TimeoutError

from health_check.contrib.redis.backends import RedisHealthCheck


class TestRedisHealthCheck:
    """Test Redis health check."""

    @mock.patch("health_check.contrib.redis.backends.getattr")
    @mock.patch("health_check.contrib.redis.backends.from_url", autospec=True)
    def test_redis_refused_connection(self, mocked_connection, mocked_getattr):
        """Test when the connection to Redis is refused."""
        mocked_getattr.return_value = "redis_url"

        # mock returns
        mocked_connection.return_value = mock.MagicMock()
        mocked_connection.return_value.__enter__.side_effect = ConnectionRefusedError(
            "Refused connection"
        )

        # instantiates the class
        redis_healthchecker = RedisHealthCheck()

        # invokes the method check_status()
        redis_healthchecker.check_status()
        assert len(redis_healthchecker.errors), 1

        # mock assertions
        mocked_connection.assert_called_once_with("redis://localhost/1")

    @mock.patch("health_check.contrib.redis.backends.getattr")
    @mock.patch("health_check.contrib.redis.backends.from_url")
    def test_redis_timeout_error(self, mocked_connection, mocked_getattr):
        """Test Redis TimeoutError."""
        mocked_getattr.return_value = "redis_url"

        # mock returns
        mocked_connection.return_value = mock.MagicMock()
        mocked_connection.return_value.__enter__.side_effect = TimeoutError(
            "Timeout Error"
        )

        # instantiates the class
        redis_healthchecker = RedisHealthCheck()

        # invokes the method check_status()
        redis_healthchecker.check_status()
        assert len(redis_healthchecker.errors), 1

        # mock assertions
        mocked_connection.assert_called_once_with("redis://localhost/1")

    @mock.patch("health_check.contrib.redis.backends.getattr")
    @mock.patch("health_check.contrib.redis.backends.from_url")
    def test_redis_con_limit_exceeded(self, mocked_connection, mocked_getattr):
        """Test Connection Limit Exceeded error."""
        mocked_getattr.return_value = "redis_url"

        # mock returns
        mocked_connection.return_value = mock.MagicMock()
        mocked_connection.return_value.__enter__.side_effect = ConnectionError(
            "Connection Error"
        )

        # instantiates the class
        redis_healthchecker = RedisHealthCheck()

        # invokes the method check_status()
        redis_healthchecker.check_status()
        assert len(redis_healthchecker.errors), 1

        # mock assertions
        mocked_connection.assert_called_once_with("redis://localhost/1")

    @mock.patch("health_check.contrib.redis.backends.getattr")
    @mock.patch("health_check.contrib.redis.backends.from_url")
    def test_redis_conn_ok(self, mocked_connection, mocked_getattr):
        """Test everything is OK."""
        mocked_getattr.return_value = "redis_url"

        # mock returns
        mocked_connection.return_value = mock.MagicMock()
        mocked_connection.return_value.__enter__.side_effect = True

        # instantiates the class
        redis_healthchecker = RedisHealthCheck()

        # invokes the method check_status()
        redis_healthchecker.check_status()
        assert len(redis_healthchecker.errors), 0

        # mock assertions
        mocked_connection.assert_called_once_with("redis://localhost/1")



import pytest

from health_check.backends import BaseHealthCheckBackend
from health_check.plugins import plugin_dir


class FakePlugin(BaseHealthCheckBackend):
    def check_status(self):
        pass


class Plugin(BaseHealthCheckBackend):
    def check_status(self):
        pass


class TestPlugin:
    @pytest.fixture(autouse=True)
    def setup(self):
        plugin_dir.reset()
        plugin_dir.register(FakePlugin)
        yield
        plugin_dir.reset()

    def test_register_plugin(self):
        assert len(plugin_dir._registry) == 1



from django.conf import settings
from django.core.cache import CacheKeyWarning, caches

from health_check.backends import BaseHealthCheckBackend
from health_check.exceptions import ServiceReturnedUnexpectedResult, ServiceUnavailable

try:
    # Exceptions thrown by Redis do not subclass builtin exceptions like ConnectionError.
    # Additionally, not only connection errors (ConnectionError -> RedisError) can be raised,
    # but also errors for time-outs (TimeoutError -> RedisError)
    # and if the backend is read-only (ReadOnlyError -> ResponseError -> RedisError).
    # Since we know what we are trying to do here, we are not picky and catch the global exception RedisError.
    from redis.exceptions import RedisError
except ModuleNotFoundError:
    # In case Redis is not installed and another cache backend is used.
    class RedisError(Exception):
        pass


class CacheBackend(BaseHealthCheckBackend):
    def __init__(self, backend="default"):
        super().__init__()
        self.backend = backend
        self.cache_key = getattr(
            settings, "HEALTHCHECK_CACHE_KEY", "djangohealthcheck_test"
        )

    def identifier(self):
        return f"Cache backend: {self.backend}"

    def check_status(self):
        cache = caches[self.backend]

        try:
            cache.set(self.cache_key, "itworks")
            if not cache.get(self.cache_key) == "itworks":
                raise ServiceUnavailable(f"Cache key {self.cache_key} does not match")
        except CacheKeyWarning as e:
            self.add_error(ServiceReturnedUnexpectedResult("Cache key warning"), e)
        except ValueError as e:
            self.add_error(ServiceReturnedUnexpectedResult("ValueError"), e)
        except (ConnectionError, RedisError) as e:
            self.add_error(ServiceReturnedUnexpectedResult("Connection Error"), e)



import logging

from health_check.exceptions import ServiceUnavailable
from health_check.storage.backends import StorageHealthCheck


class S3Boto3StorageHealthCheck(StorageHealthCheck):
    """
    Tests the status of a `S3BotoStorage` file storage backend.

    S3BotoStorage is included in the `django-storages` package
    and recommended by for example Amazon and Heroku for Django
    static and media file storage on cloud platforms.

    ``django-storages`` can be found at https://git.io/v1lGx
    ``S3Boto3Storage`` can be found at
        https://github.com/jschneier/django-storages/blob/master/storages/backends/s3boto3.py
    """

    logger = logging.getLogger(__name__)
    storage = "storages.backends.s3boto3.S3Boto3Storage"
    storage_alias = "default"

    def check_delete(self, file_name):
        storage = self.get_storage()
        if not storage.exists(file_name):
            raise ServiceUnavailable("File does not exist")
        storage.delete(file_name)



import re

from django.http import HttpResponse, JsonResponse
from django.utils.decorators import method_decorator
from django.views.decorators.cache import never_cache
from django.views.generic import TemplateView

from health_check.mixins import CheckMixin


class MediaType:
    """
    Sortable object representing HTTP's accept header.

    .. seealso:: https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Accept
    """

    pattern = re.compile(
        r"""
          ^
          (?P<mime_type>
            (\w+|\*)                      # Media type, or wildcard
            /
            ([\w\d\-+.]+|\*)              # subtype, or wildcard
          )
          (
            \s*;\s*                       # parameter separator with optional whitespace
            q=                            # q is expected to be the first parameter, by RFC2616
            (?P<weight>
              1([.]0{1,3})?               # 1 with up to three digits of precision
              |
              0([.]\d{1,3})?              # 0.000 to 0.999 with optional precision
            )
          )?
          (
            \s*;\s*                       # parameter separator with optional whitespace
            [-!#$%&'*+.^_`|~0-9a-zA-Z]+   # any token from legal characters
            =
            [-!#$%&'*+.^_`|~0-9a-zA-Z]+   # any value from legal characters
          )*
          $
        """,
        re.VERBOSE,
    )

    def __init__(self, mime_type, weight=1.0):
        self.mime_type = mime_type
        self.weight = float(weight)

    @classmethod
    def from_string(cls, value):
        """Return single instance parsed from given accept header string."""
        match = cls.pattern.search(value)
        if match is None:
            raise ValueError('"%s" is not a valid media type' % value)
        try:
            return cls(match.group("mime_type"), float(match.group("weight") or 1))
        except ValueError:
            return cls(value)

    @classmethod
    def parse_header(cls, value="*/*"):
        """Parse HTTP accept header and return instances sorted by weight."""
        yield from sorted(
            (
                cls.from_string(token.strip())
                for token in value.split(",")
                if token.strip()
            ),
            reverse=True,
        )

    def __str__(self):
        return "%s; q=%s" % (self.mime_type, self.weight)

    def __repr__(self):
        return "%s: %s" % (type(self).__name__, self.__str__())

    def __eq__(self, other):
        return self.weight == other.weight and self.mime_type == other.mime_type

    def __lt__(self, other):
        return self.weight.__lt__(other.weight)


class MainView(CheckMixin, TemplateView):
    template_name = "health_check/index.html"

    @method_decorator(never_cache)
    def get(self, request, *args, **kwargs):
        subset = kwargs.get("subset", None)
        health_check_has_error = self.check(subset)
        status_code = 500 if health_check_has_error else 200
        format_override = request.GET.get("format")

        if format_override == "json":
            return self.render_to_response_json(
                self.filter_plugins(subset=subset), status_code
            )

        accept_header = request.META.get("HTTP_ACCEPT", "*/*")
        for media in MediaType.parse_header(accept_header):
            if media.mime_type in (
                "text/html",
                "application/xhtml+xml",
                "text/*",
                "*/*",
            ):
                context = self.get_context_data(**kwargs)
                return self.render_to_response(context, status=status_code)
            elif media.mime_type in ("application/json", "application/*"):
                return self.render_to_response_json(
                    self.filter_plugins(subset=subset), status_code
                )
        return HttpResponse(
            "Not Acceptable: Supported content types: text/html, application/json",
            status=406,
            content_type="text/plain",
        )

    def get_context_data(self, **kwargs):
        subset = kwargs.get("subset", None)
        return {
            **super().get_context_data(**kwargs),
            "plugins": self.filter_plugins(subset=subset).values(),
        }

    def render_to_response_json(self, plugins, status):
        return JsonResponse(
            {
                str(plugin_identifier): str(p.pretty_status())
                for plugin_identifier, p in plugins.items()
            },
            status=status,
        )



class AlreadyRegistered(Exception):
    pass


class NotRegistered(Exception):
    pass


class HealthCheckPluginDirectory:
    """Django health check registry."""

    def __init__(self):
        self._registry = []  # plugin_class class -> plugin options

    def reset(self):
        """Reset registry state, e.g. for testing purposes."""
        self._registry = []

    def register(self, plugin, **options):
        """Add the given plugin from the registry."""
        # Instantiate the admin class to save in the registry
        self._registry.append((plugin, options))


plugin_dir = HealthCheckPluginDirectory()



from unittest.mock import patch

from django.core.cache.backends.base import BaseCache, CacheKeyWarning
from django.test import TestCase

from health_check.cache.backends import CacheBackend


# A Mock version of the cache to use for testing
class MockCache(BaseCache):
    """
    A Mock Cache used for testing.
    set_works - set to False to make the mocked set method fail, but not raise
    set_raises - The Exception to be raised when set() is called, if any
    """

    key = None
    value = None
    set_works = None
    set_raises = None

    def __init__(self, set_works=True, set_raises=None):
        super(MockCache, self).__init__(params={})
        self.set_works = set_works
        self.set_raises = set_raises

    def set(self, key, value, *args, **kwargs):
        if self.set_raises is not None:
            raise self.set_raises
        elif self.set_works:
            self.key = key
            self.value = value
        else:
            self.key = key
            self.value = None

    def get(self, key, *args, **kwargs):
        if key == self.key:
            return self.value
        else:
            return None


class HealthCheckCacheTests(TestCase):
    """
    Tests health check behavior with a mocked cache backend.
    Ensures check_status returns/raises the expected result when the cache works, fails, or raises exceptions.
    """

    @patch("health_check.cache.backends.caches", dict(default=MockCache()))
    def test_check_status_working(self):
        cache_backend = CacheBackend()
        cache_backend.run_check()
        self.assertFalse(cache_backend.errors)

    @patch(
        "health_check.cache.backends.caches",
        dict(default=MockCache(), broken=MockCache(set_works=False)),
    )
    def test_multiple_backends_check_default(self):
        # default backend works while other is broken
        cache_backend = CacheBackend("default")
        cache_backend.run_check()
        self.assertFalse(cache_backend.errors)

    @patch(
        "health_check.cache.backends.caches",
        dict(default=MockCache(), broken=MockCache(set_works=False)),
    )
    def test_multiple_backends_check_broken(self):
        cache_backend = CacheBackend("broken")
        cache_backend.run_check()
        self.assertTrue(cache_backend.errors)
        self.assertIn("does not match", cache_backend.pretty_status())

    # check_status should raise ServiceUnavailable when values at cache key do not match
    @patch(
        "health_check.cache.backends.caches", dict(default=MockCache(set_works=False))
    )
    def test_set_fails(self):
        cache_backend = CacheBackend()
        cache_backend.run_check()
        self.assertTrue(cache_backend.errors)
        self.assertIn("does not match", cache_backend.pretty_status())

    # check_status should catch generic exceptions raised by set and convert to ServiceUnavailable
    @patch(
        "health_check.cache.backends.caches",
        dict(default=MockCache(set_raises=Exception)),
    )
    def test_set_raises_generic(self):
        cache_backend = CacheBackend()
        with self.assertRaises(Exception):
            cache_backend.run_check()

    # check_status should catch CacheKeyWarning and convert to ServiceReturnedUnexpectedResult
    @patch(
        "health_check.cache.backends.caches",
        dict(default=MockCache(set_raises=CacheKeyWarning)),
    )
    def test_set_raises_cache_key_warning(self):
        cache_backend = CacheBackend()
        cache_backend.check_status()
        cache_backend.run_check()
        self.assertIn(
            "unexpected result: Cache key warning", cache_backend.pretty_status()
        )



from celery.app import default_app as app
from django.conf import settings

from health_check.backends import BaseHealthCheckBackend
from health_check.exceptions import ServiceUnavailable


class CeleryPingHealthCheck(BaseHealthCheckBackend):
    CORRECT_PING_RESPONSE = {"ok": "pong"}

    def check_status(self):
        timeout = getattr(settings, "HEALTHCHECK_CELERY_PING_TIMEOUT", 1)

        try:
            ping_result = app.control.ping(timeout=timeout)
        except IOError as e:
            self.add_error(ServiceUnavailable("IOError"), e)
        except NotImplementedError as exc:
            self.add_error(
                ServiceUnavailable(
                    "NotImplementedError: Make sure CELERY_RESULT_BACKEND is set"
                ),
                exc,
            )
        except BaseException as exc:
            self.add_error(ServiceUnavailable("Unknown error"), exc)
        else:
            if not ping_result:
                self.add_error(
                    ServiceUnavailable("Celery workers unavailable"),
                )
            else:
                self._check_ping_result(ping_result)

    def _check_ping_result(self, ping_result):
        active_workers = []

        for result in ping_result:
            worker, response = list(result.items())[0]
            if response != self.CORRECT_PING_RESPONSE:
                self.add_error(
                    ServiceUnavailable(
                        f"Celery worker {worker} response was incorrect"
                    ),
                )
                continue
            active_workers.append(worker)

        if not self.errors:
            self._check_active_queues(active_workers)

    def _check_active_queues(self, active_workers):
        defined_queues = app.conf.CELERY_QUEUES

        if not defined_queues:
            return

        defined_queues = set([queue.name for queue in defined_queues])
        active_queues = set()

        for queues in app.control.inspect(active_workers).active_queues().values():
            active_queues.update([queue.get("name") for queue in queues])

        for queue in defined_queues.difference(active_queues):
            self.add_error(
                ServiceUnavailable(f"No worker for Celery task queue {queue}"),
            )



import locale
import socket

import psutil

from health_check.backends import BaseHealthCheckBackend
from health_check.conf import HEALTH_CHECK
from health_check.exceptions import ServiceReturnedUnexpectedResult, ServiceWarning

host = socket.gethostname()

DISK_USAGE_MAX = HEALTH_CHECK["DISK_USAGE_MAX"]
MEMORY_MIN = HEALTH_CHECK["MEMORY_MIN"]


class DiskUsage(BaseHealthCheckBackend):
    def check_status(self):
        try:
            du = psutil.disk_usage("/")
            if DISK_USAGE_MAX and du.percent >= DISK_USAGE_MAX:
                raise ServiceWarning(
                    "{host} {percent}% disk usage exceeds {disk_usage}%".format(
                        host=host, percent=du.percent, disk_usage=DISK_USAGE_MAX
                    )
                )
        except ValueError as e:
            self.add_error(ServiceReturnedUnexpectedResult("ValueError"), e)


class MemoryUsage(BaseHealthCheckBackend):
    def check_status(self):
        try:
            memory = psutil.virtual_memory()
            if MEMORY_MIN and memory.available < (MEMORY_MIN * 1024 * 1024):
                locale.setlocale(locale.LC_ALL, "")
                avail = "{:n}".format(int(memory.available / 1024 / 1024))
                threshold = "{:n}".format(MEMORY_MIN)
                raise ServiceWarning(
                    "{host} {avail} MB available RAM below {threshold} MB".format(
                        host=host, avail=avail, threshold=threshold
                    )
                )
        except ValueError as e:
            self.add_error(ServiceReturnedUnexpectedResult("ValueError"), e)



from django.apps import AppConfig
from django.conf import settings

from health_check.plugins import plugin_dir


class HealthCheckConfig(AppConfig):
    name = "health_check.cache"

    def ready(self):
        from .backends import CacheBackend

        for backend in settings.CACHES:
            plugin_dir.register(CacheBackend, backend=backend)



from django.utils.translation import gettext_lazy as _  # noqa: N812


class HealthCheckException(Exception):
    message_type = _("unknown error")

    def __init__(self, message):
        self.message = message

    def __str__(self):
        return "%s: %s" % (self.message_type, self.message)


class ServiceWarning(HealthCheckException):
    """
    Warning of service misbehavior.

    If the ``HEALTH_CHECK['WARNINGS_AS_ERRORS']`` is set to ``False``,
    these exceptions will not case a 500 status response.
    """

    message_type = _("warning")


class ServiceUnavailable(HealthCheckException):
    message_type = _("unavailable")


class ServiceReturnedUnexpectedResult(HealthCheckException):
    message_type = _("unexpected result")



import logging
from timeit import default_timer as timer

from django.utils.translation import gettext_lazy as _  # noqa: N812

from health_check.exceptions import HealthCheckException

logger = logging.getLogger("health-check")


class BaseHealthCheckBackend:
    critical_service = True
    """
    Define if service is critical to the operation of the site.

    If set to ``True`` service failures return 500 response code on the
    health check endpoint.
    """

    def __init__(self):
        self.errors = []

    def check_status(self):
        raise NotImplementedError

    def run_check(self):
        start = timer()
        self.errors = []
        try:
            self.check_status()
        except HealthCheckException as e:
            self.add_error(e, e)
        except BaseException:
            logger.exception("Unexpected Error!")
            raise
        finally:
            self.time_taken = timer() - start

    def add_error(self, error, cause=None):
        if isinstance(error, HealthCheckException):
            pass
        elif isinstance(error, str):
            msg = error
            error = HealthCheckException(msg)
        else:
            msg = _("unknown error")
            error = HealthCheckException(msg)
        if isinstance(cause, BaseException):
            logger.exception(str(error))
        else:
            logger.error(str(error))
        self.errors.append(error)

    def pretty_status(self):
        if self.errors:
            return "\n".join(str(e) for e in self.errors)
        return _("working")

    @property
    def status(self):
        return int(not self.errors)

    def identifier(self):
        return self.__class__.__name__



from unittest.mock import patch

from django.db.migrations import Migration
from django.test import TestCase

from health_check.contrib.migrations.backends import MigrationsHealthCheck


class MockMigration(Migration):
    ...


class TestMigrationsHealthCheck(TestCase):
    def test_check_status_work(self):
        with patch(
            "health_check.contrib.migrations.backends.MigrationsHealthCheck.get_migration_plan",
            return_value=[],
        ):
            backend = MigrationsHealthCheck()
            backend.run_check()
            self.assertFalse(backend.errors)

    def test_check_status_raises_error_if_there_are_migrations(self):
        with patch(
            "health_check.contrib.migrations.backends.MigrationsHealthCheck.get_migration_plan",
            return_value=[(MockMigration, False)],
        ):
            backend = MigrationsHealthCheck()
            backend.run_check()
            self.assertTrue(backend.errors)



from unittest.mock import patch

import pytest
from django.apps import apps
from django.conf import settings

from health_check.contrib.celery_ping.apps import HealthCheckConfig
from health_check.contrib.celery_ping.backends import CeleryPingHealthCheck


class TestCeleryPingHealthCheck:
    CELERY_APP_CONTROL_PING = (
        "health_check.contrib.celery_ping.backends.app.control.ping"
    )
    CELERY_APP_CONTROL_INSPECT_ACTIVE_QUEUES = (
        "health_check.contrib.celery_ping.backends.app.control.inspect.active_queues"
    )

    @pytest.fixture
    def health_check(self):
        return CeleryPingHealthCheck()

    def test_check_status_doesnt_add_errors_when_ping_successful(self, health_check):
        celery_worker = "celery@4cc150a7b49b"

        with patch(
            self.CELERY_APP_CONTROL_PING,
            return_value=[
                {celery_worker: CeleryPingHealthCheck.CORRECT_PING_RESPONSE},
                {f"{celery_worker}-2": CeleryPingHealthCheck.CORRECT_PING_RESPONSE},
            ],
        ), patch(
            self.CELERY_APP_CONTROL_INSPECT_ACTIVE_QUEUES,
            return_value={
                celery_worker: [
                    {"name": queue.name} for queue in settings.CELERY_QUEUES
                ]
            },
        ):
            health_check.check_status()

            assert not health_check.errors

    def test_check_status_reports_errors_if_ping_responses_are_incorrect(
        self, health_check
    ):
        with patch(
            self.CELERY_APP_CONTROL_PING,
            return_value=[
                {"celery1@4cc150a7b49b": CeleryPingHealthCheck.CORRECT_PING_RESPONSE},
                {"celery2@4cc150a7b49b": {}},
                {"celery3@4cc150a7b49b": {"error": "pong"}},
            ],
        ):
            health_check.check_status()

            assert len(health_check.errors) == 2

    def test_check_status_adds_errors_when_ping_successfull_but_not_all_defined_queues_have_consumers(
        self,
        health_check,
    ):
        celery_worker = "celery@4cc150a7b49b"
        queues = list(settings.CELERY_QUEUES)

        with patch(
            self.CELERY_APP_CONTROL_PING,
            return_value=[{celery_worker: CeleryPingHealthCheck.CORRECT_PING_RESPONSE}],
        ), patch(
            self.CELERY_APP_CONTROL_INSPECT_ACTIVE_QUEUES,
            return_value={celery_worker: [{"name": queues.pop().name}]},
        ):
            health_check.check_status()

            assert len(health_check.errors) == len(queues)

    @pytest.mark.parametrize(
        "exception_to_raise",
        [
            IOError,
            TimeoutError,
        ],
    )
    def test_check_status_add_error_when_io_error_raised_from_ping(
        self, exception_to_raise, health_check
    ):
        with patch(self.CELERY_APP_CONTROL_PING, side_effect=exception_to_raise):
            health_check.check_status()

            assert len(health_check.errors) == 1
            assert "ioerror" in health_check.errors[0].message.lower()

    @pytest.mark.parametrize(
        "exception_to_raise", [ValueError, SystemError, IndexError, MemoryError]
    )
    def test_check_status_add_error_when_any_exception_raised_from_ping(
        self, exception_to_raise, health_check
    ):
        with patch(self.CELERY_APP_CONTROL_PING, side_effect=exception_to_raise):
            health_check.check_status()

            assert len(health_check.errors) == 1
            assert health_check.errors[0].message.lower() == "unknown error"

    def test_check_status_when_raised_exception_notimplementederror(self, health_check):
        expected_error_message = (
            "notimplementederror: make sure celery_result_backend is set"
        )

        with patch(self.CELERY_APP_CONTROL_PING, side_effect=NotImplementedError):
            health_check.check_status()

            assert len(health_check.errors) == 1
            assert health_check.errors[0].message.lower() == expected_error_message

    @pytest.mark.parametrize("ping_result", [None, list()])
    def test_check_status_add_error_when_ping_result_failed(
        self, ping_result, health_check
    ):
        with patch(self.CELERY_APP_CONTROL_PING, return_value=ping_result):
            health_check.check_status()

            assert len(health_check.errors) == 1
            assert "workers unavailable" in health_check.errors[0].message.lower()


class TestCeleryPingHealthCheckApps:
    def test_apps(self):
        assert HealthCheckConfig.name == "health_check.contrib.celery_ping"

        celery_ping = apps.get_app_config("celery_ping")
        assert celery_ping.name == "health_check.contrib.celery_ping"



from django.conf import settings

HEALTH_CHECK = getattr(settings, "HEALTH_CHECK", {})
HEALTH_CHECK.setdefault("DISK_USAGE_MAX", 90)
HEALTH_CHECK.setdefault("MEMORY_MIN", 100)
HEALTH_CHECK.setdefault("WARNINGS_AS_ERRORS", True)
HEALTH_CHECK.setdefault("SUBSETS", {})
HEALTH_CHECK.setdefault("DISABLE_THREADING", False)



import logging

from amqp.exceptions import AccessRefused
from django.conf import settings
from kombu import Connection

from health_check.backends import BaseHealthCheckBackend
from health_check.exceptions import ServiceUnavailable

logger = logging.getLogger(__name__)


class RabbitMQHealthCheck(BaseHealthCheckBackend):
    """Health check for RabbitMQ."""

    namespace = None

    def check_status(self):
        """Check RabbitMQ service by opening and closing a broker channel."""
        logger.debug("Checking for a broker_url on django settings...")

        broker_url_setting_key = (
            f"{self.namespace}_BROKER_URL" if self.namespace else "BROKER_URL"
        )
        broker_url = getattr(settings, broker_url_setting_key, None)

        logger.debug("Got %s as the broker_url. Connecting to rabbit...", broker_url)

        logger.debug("Attempting to connect to rabbit...")
        try:
            # conn is used as a context to release opened resources later
            with Connection(broker_url) as conn:
                conn.connect()  # exceptions may be raised upon calling connect
        except ConnectionRefusedError as e:
            self.add_error(
                ServiceUnavailable(
                    "Unable to connect to RabbitMQ: Connection was refused."
                ),
                e,
            )

        except AccessRefused as e:
            self.add_error(
                ServiceUnavailable(
                    "Unable to connect to RabbitMQ: Authentication error."
                ),
                e,
            )

        except IOError as e:
            self.add_error(ServiceUnavailable("IOError"), e)

        except BaseException as e:
            self.add_error(ServiceUnavailable("Unknown error"), e)
        else:
            logger.debug("Connection established. RabbitMQ is healthy.")



import logging

from django.conf import settings
from django.db import DEFAULT_DB_ALIAS, DatabaseError, connections
from django.db.migrations.executor import MigrationExecutor

from health_check.backends import BaseHealthCheckBackend
from health_check.exceptions import ServiceUnavailable

logger = logging.getLogger(__name__)


class MigrationsHealthCheck(BaseHealthCheckBackend):
    def get_migration_plan(self, executor):
        return executor.migration_plan(executor.loader.graph.leaf_nodes())

    def check_status(self):
        db_alias = getattr(settings, "HEALTHCHECK_MIGRATIONS_DB", DEFAULT_DB_ALIAS)
        try:
            executor = MigrationExecutor(connections[db_alias])
            plan = self.get_migration_plan(executor)
            if plan:
                self.add_error(ServiceUnavailable("There are migrations to apply"))
        except DatabaseError as e:
            self.add_error(ServiceUnavailable("Database is not ready"), e)
        except Exception as e:
            self.add_error(ServiceUnavailable("Unexpected error"), e)



import logging
from io import StringIO

import pytest

from health_check.backends import BaseHealthCheckBackend
from health_check.exceptions import HealthCheckException


class TestBaseHealthCheckBackend:
    def test_run_check(self):
        with pytest.raises(NotImplementedError):
            BaseHealthCheckBackend().run_check()

    def test_identifier(self):
        assert BaseHealthCheckBackend().identifier() == "BaseHealthCheckBackend"

        class MyHeathCheck(BaseHealthCheckBackend):
            pass

        assert MyHeathCheck().identifier() == "MyHeathCheck"

        class MyHeathCheck(BaseHealthCheckBackend):
            foo = "bar"

            def identifier(self):
                return self.foo

        assert MyHeathCheck().identifier() == "bar"

    def test_status(self):
        ht = BaseHealthCheckBackend()
        assert ht.status == 1
        ht.errors = [1]
        assert ht.status == 0

    def test_pretty_status(self):
        ht = BaseHealthCheckBackend()
        assert ht.pretty_status() == "working"
        ht.errors = ["foo"]
        assert ht.pretty_status() == "foo"
        ht.errors.append("bar")
        assert ht.pretty_status() == "foo\nbar"
        ht.errors.append(123)
        assert ht.pretty_status() == "foo\nbar\n123"

    def test_add_error(self):
        ht = BaseHealthCheckBackend()
        e = HealthCheckException("foo")
        ht.add_error(e)
        assert ht.errors[0] is e

        ht = BaseHealthCheckBackend()
        ht.add_error("bar")
        assert isinstance(ht.errors[0], HealthCheckException)
        assert str(ht.errors[0]) == "unknown error: bar"

        ht = BaseHealthCheckBackend()
        ht.add_error(type)
        assert isinstance(ht.errors[0], HealthCheckException)
        assert str(ht.errors[0]) == "unknown error: unknown error"

    def test_add_error_cause(self):
        ht = BaseHealthCheckBackend()
        logger = logging.getLogger("health-check")
        with StringIO() as stream:
            stream_handler = logging.StreamHandler(stream)
            logger.addHandler(stream_handler)
            try:
                raise Exception("bar")
            except Exception as e:
                ht.add_error("foo", e)

            stream.seek(0)
            log = stream.read()
            assert "foo" in log
            assert "bar" in log
            assert "Traceback" in log
            assert "Exception: bar" in log
            logger.removeHandler(stream_handler)

        with StringIO() as stream:
            stream_handler = logging.StreamHandler(stream)
            logger.addHandler(stream_handler)
            try:
                raise Exception("bar")
            except Exception:
                ht.add_error("foo")

            stream.seek(0)
            log = stream.read()
            assert "foo" in log
            assert "bar" not in log
            assert "Traceback" not in log
            assert "Exception: bar" not in log
            logger.removeHandler(stream_handler)



import warnings

from celery import current_app
from django.apps import AppConfig
from django.conf import settings

from health_check.plugins import plugin_dir


class HealthCheckConfig(AppConfig):
    name = "health_check.contrib.celery"

    def ready(self):
        from .backends import CeleryHealthCheck

        if hasattr(settings, "HEALTHCHECK_CELERY_TIMEOUT"):
            warnings.warn(
                "HEALTHCHECK_CELERY_TIMEOUT is depricated and may be removed in the "
                "future. Please use HEALTHCHECK_CELERY_RESULT_TIMEOUT and "
                "HEALTHCHECK_CELERY_QUEUE_TIMEOUT instead.",
                DeprecationWarning,
            )

        for queue in current_app.amqp.queues:
            celery_class_name = "CeleryHealthCheck" + queue.title()

            celery_class = type(
                celery_class_name, (CeleryHealthCheck,), {"queue": queue}
            )
            plugin_dir.register(celery_class)



# Generated by Django 1.10.1 on 2016-09-26 18:46
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    replaces = [
        ("health_check_db", "0001_initial"),
    ]

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="TestModel",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=128)),
            ],
            options={
                "db_table": "health_check_db_testmodel",
            },
        ),
    ]



from django.db import DatabaseError, IntegrityError

from health_check.backends import BaseHealthCheckBackend
from health_check.exceptions import ServiceReturnedUnexpectedResult, ServiceUnavailable

from .models import TestModel


class DatabaseBackend(BaseHealthCheckBackend):
    def check_status(self):
        try:
            obj = TestModel.objects.create(title="test")
            obj.title = "newtest"
            obj.save()
            obj.delete()
        except IntegrityError:
            raise ServiceReturnedUnexpectedResult("Integrity Error")
        except DatabaseError:
            raise ServiceUnavailable("Database error")



from unittest import mock

from amqp.exceptions import AccessRefused

from health_check.contrib.rabbitmq.backends import RabbitMQHealthCheck


class TestRabbitMQHealthCheck:
    """Test RabbitMQ health check."""

    @mock.patch("health_check.contrib.rabbitmq.backends.getattr")
    @mock.patch("health_check.contrib.rabbitmq.backends.Connection")
    def test_broker_refused_connection(self, mocked_connection, mocked_getattr):
        """Test when the connection to RabbitMQ is refused."""
        mocked_getattr.return_value = "broker_url"

        conn_exception = ConnectionRefusedError("Refused connection")

        # mock returns
        mocked_conn = mock.MagicMock()
        mocked_connection.return_value.__enter__.return_value = mocked_conn
        mocked_conn.connect.side_effect = conn_exception

        # instantiates the class
        rabbitmq_healthchecker = RabbitMQHealthCheck()

        # invokes the method check_status()
        rabbitmq_healthchecker.check_status()
        assert len(rabbitmq_healthchecker.errors), 1

        # mock assertions
        mocked_connection.assert_called_once_with("broker_url")

    @mock.patch("health_check.contrib.rabbitmq.backends.getattr")
    @mock.patch("health_check.contrib.rabbitmq.backends.Connection")
    def test_broker_auth_error(self, mocked_connection, mocked_getattr):
        """Test that the connection to RabbitMQ has an authentication error."""
        mocked_getattr.return_value = "broker_url"

        conn_exception = AccessRefused("Refused connection")

        # mock returns
        mocked_conn = mock.MagicMock()
        mocked_connection.return_value.__enter__.return_value = mocked_conn
        mocked_conn.connect.side_effect = conn_exception

        # instantiates the class
        rabbitmq_healthchecker = RabbitMQHealthCheck()

        # invokes the method check_status()
        rabbitmq_healthchecker.check_status()
        assert len(rabbitmq_healthchecker.errors), 1

        # mock assertions
        mocked_connection.assert_called_once_with("broker_url")

    @mock.patch("health_check.contrib.rabbitmq.backends.getattr")
    @mock.patch("health_check.contrib.rabbitmq.backends.Connection")
    def test_broker_connection_upon_none_url(self, mocked_connection, mocked_getattr):
        """Thest when the connection to RabbitMQ has no ``broker_url``."""
        mocked_getattr.return_value = None
        # if the variable BROKER_URL is not set, AccessRefused exception is raised
        conn_exception = AccessRefused("Refused connection")

        # mock returns
        mocked_conn = mock.MagicMock()
        mocked_connection.return_value.__enter__.return_value = mocked_conn
        mocked_conn.connect.side_effect = conn_exception

        # instantiates the class
        rabbitmq_healthchecker = RabbitMQHealthCheck()

        # invokes the method check_status()
        rabbitmq_healthchecker.check_status()
        assert len(rabbitmq_healthchecker.errors), 1

        # mock assertions
        mocked_connection.assert_called_once_with(None)



import sys

from django.core.management.base import BaseCommand
from django.http import Http404

from health_check.mixins import CheckMixin


class Command(CheckMixin, BaseCommand):
    help = "Run health checks and exit 0 if everything went well."

    def add_arguments(self, parser):
        parser.add_argument("-s", "--subset", type=str, nargs=1)

    def handle(self, *args, **options):
        # perform all checks
        subset = options.get("subset", [])
        subset = subset[0] if subset else None
        try:
            errors = self.check(subset=subset)
        except Http404 as e:
            self.stdout.write(str(e))
            sys.exit(1)

        for plugin_identifier, plugin in self.filter_plugins(subset=subset).items():
            style_func = self.style.SUCCESS if not plugin.errors else self.style.ERROR
            self.stdout.write(
                "{:<24} ... {}\n".format(
                    plugin_identifier, style_func(plugin.pretty_status())
                )
            )

        if errors:
            sys.exit(1)



from celery import current_app
from django.conf import settings

from health_check.contrib.celery.backends import CeleryHealthCheck
from health_check.contrib.celery_ping.backends import CeleryPingHealthCheck
from health_check.plugins import plugin_dir


class TestAutoDiscover:
    def test_autodiscover(self):
        health_check_plugins = list(
            filter(
                lambda x: x.startswith("health_check.") and "celery" not in x,
                settings.INSTALLED_APPS,
            )
        )

        non_celery_plugins = [
            x
            for x in plugin_dir._registry
            if not issubclass(x[0], (CeleryHealthCheck, CeleryPingHealthCheck))
        ]

        # The number of installed apps excluding celery should equal to all plugins except celery
        assert len(non_celery_plugins) == len(health_check_plugins)

    def test_discover_celery_queues(self):
        celery_plugins = [
            x for x in plugin_dir._registry if issubclass(x[0], CeleryHealthCheck)
        ]
        assert len(celery_plugins) == len(current_app.amqp.queues)



from django.urls import path

from health_check.views import MainView

app_name = "health_check"

urlpatterns = [
    path("", MainView.as_view(), name="health_check_home"),
    path("<str:subset>/", MainView.as_view(), name="health_check_subset"),
]



from django.apps import AppConfig

from health_check.plugins import plugin_dir


class HealthCheckConfig(AppConfig):
    name = "health_check.contrib.rabbitmq"

    def ready(self):
        from .backends import RabbitMQHealthCheck

        plugin_dir.register(RabbitMQHealthCheck)



import uuid

import django
from django.core.files.base import ContentFile
from django.core.files.storage import default_storage

if django.VERSION >= (4, 2):
    from django.core.files.storage import InvalidStorageError, storages
else:
    from django.core.files.storage import get_storage_class

from health_check.backends import BaseHealthCheckBackend
from health_check.exceptions import ServiceUnavailable


class StorageHealthCheck(BaseHealthCheckBackend):
    """
    Tests the status of a `StorageBackend`.

    Can be extended to test any storage backend by subclassing:

        class MyStorageHealthCheck(StorageHealthCheck):
            storage = 'some.other.StorageBackend'
        plugin_dir.register(MyStorageHealthCheck)

    storage must be either a string pointing to a storage class
    (e.g 'django.core.files.storage.FileSystemStorage') or a Storage instance.
    """

    storage_alias = None
    storage = None

    def get_storage(self):
        if django.VERSION >= (4, 2):
            try:
                return storages[self.storage_alias]
            except InvalidStorageError:
                return None
        else:
            if isinstance(self.storage, str):
                return get_storage_class(self.storage)()
            else:
                return self.storage

    def get_file_name(self):
        return "health_check_storage_test/test-%s.txt" % uuid.uuid4()

    def get_file_content(self):
        return b"this is the healthtest file content"

    def check_save(self, file_name, file_content):
        storage = self.get_storage()
        # save the file
        file_name = storage.save(file_name, ContentFile(content=file_content))
        # read the file and compare
        if not storage.exists(file_name):
            raise ServiceUnavailable("File does not exist")
        with storage.open(file_name) as f:
            if not f.read() == file_content:
                raise ServiceUnavailable("File content does not match")
        return file_name

    def check_delete(self, file_name):
        storage = self.get_storage()
        # delete the file and make sure it is gone
        storage.delete(file_name)
        if storage.exists(file_name):
            raise ServiceUnavailable("File was not deleted")

    def check_status(self):
        try:
            # write the file to the storage backend
            file_name = self.get_file_name()
            file_content = self.get_file_content()
            file_name = self.check_save(file_name, file_content)
            self.check_delete(file_name)
            return True
        except Exception as e:
            raise ServiceUnavailable("Unknown exception") from e


class DefaultFileStorageHealthCheck(StorageHealthCheck):
    storage_alias = "default"
    storage = default_storage



from unittest.mock import patch

from django.db import DatabaseError, IntegrityError
from django.db.models import Model
from django.test import TestCase

from health_check.db.backends import DatabaseBackend


class MockDBModel(Model):
    """
    A Mock database used for testing.
    error_thrown - The Exception to be raised when save() is called, if any
    """

    error_thrown = None

    def __init__(self, error_thrown=None, *args, **kwargs):
        super(MockDBModel, self).__init__(*args, **kwargs)
        self.error_thrown = error_thrown

    def save(self, *args, **kwargs):
        if self.error_thrown is not None:
            raise self.error_thrown
        else:
            return True

    def delete(self, *args, **kwargs):
        return True


def raise_(ex):
    raise ex


class HealthCheckDatabaseTests(TestCase):
    """
    Tests health check behavior with a mocked database backend.
    Ensures check_status returns/raises the expected result when the database works or raises exceptions.
    """

    @patch(
        "health_check.db.backends.TestModel.objects.create",
        lambda title=None: MockDBModel(),
    )
    def test_check_status_works(self):
        db_backend = DatabaseBackend()
        db_backend.check_status()
        self.assertFalse(db_backend.errors)

    @patch(
        "health_check.db.backends.TestModel.objects.create",
        lambda title=None: raise_(IntegrityError),
    )
    def test_raise_integrity_error(self):
        db_backend = DatabaseBackend()
        db_backend.run_check()
        self.assertTrue(db_backend.errors)
        self.assertIn("unexpected result: Integrity Error", db_backend.pretty_status())

    @patch(
        "health_check.db.backends.TestModel.objects.create",
        lambda title=None: MockDBModel(error_thrown=DatabaseError),
    )
    def test_raise_database_error(self):
        db_backend = DatabaseBackend()
        db_backend.run_check()
        self.assertTrue(db_backend.errors)
        self.assertIn("unavailable: Database error", db_backend.pretty_status())

    @patch(
        "health_check.db.backends.TestModel.objects.create",
        lambda title=None: MockDBModel(error_thrown=Exception),
    )
    def test_raise_exception(self):
        db_backend = DatabaseBackend()
        with self.assertRaises(Exception):
            db_backend.run_check()
